# coding:utf-8

from urllib.parse import urljoin

__project__ = "xpw"
__version__ = "0.1.0"
__description__ = "Securely manage user passwords and authentication."
__urlhome__ = "https://github.com/bondbox/xpw/"
__urlcode__ = __urlhome__
__urldocs__ = __urlhome__
__urlbugs__ = urljoin(__urlhome__, "issues")

# author
__author__ = "Mingzhe Zou"
__author_email__ = "zoumingzhe@outlook.com"
